# -*- coding: utf-8 -*-

""" Panohead remote control.

License
=======

 - B{Papywizard} (U{http://www.papywizard.org}) is Copyright:
  - (C) 2007-2011 Frédéric Mantegazza

This software is governed by the B{CeCILL} license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
U{http://www.cecill.info}.

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

Module purpose
==============

Model

The commands are the

Implements
==========

 - AbstractCommand
 - MoveHeadCommand
 - PauseShootingCommand
 - ResumeShootingCommand
 - StopShootingCommand
 - WaitCommand

@author: Frédéric Mantegazza
@copyright: (C) 2007-2011 Frédéric Mantegazza
@license: CeCILL
"""

__revision__ = "$Id:$"

from PyQt4 import QtCore


class AbstractCommand(QtCore.QObject):
    """ Abstract command class.

    This class handles all common commands features.
    """
    def __init__(self, name):
        """ Init the object.

        @param name: name of the command
        @type name: str
        """
        QtCore.QObject.__init__(self)

        self._name = name

        self._init()

    def _init(self):
        """ Additional init.
        """
        raise NotImplementedError("AbstractCommand._init() must be overloaded")

    def execute(self):
        """ Execute the command.
        """
        raise NotImplementedError("AbstractCommand.execute() must be overloaded")


class MoveHeadCommand(AbstractCommand):
    """ Command to move the head.
    """
    def _init(self):
        self.__yaw = self.__pitch = None

    def setPosition(self, yaw, pitch):
        """ Set the position to reach.

        @param yaw: yaw position to reach, in °
        @type yaw: float

        @param pitch: pitch position to reach, in °
        @type pitch: float

        @raise PapywizardError: no position has been set
        """
        self.__yaw = yaw
        self.__pitch = pitch

    def getPosition(self):
        """ Get position to reach.

        @return: position to reach (yaw, pitch)
        @rtype: tuple of float
        """
        if self.__yaw is None or self.__pitc is None:
            raise PapywizardError("No position has been set")
        return self.__yaw, self.__pitch
